unit dnDBPxM;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Components TDBPxEdit, TDBPxCombo, new properties:
    PxMask: loaded from corresponding TPx...Field;

Bugs fixed:
1.02 :
  stupid bug: In D2 method LoadPxMasks didn't work, because I changed code a bit
  (working with D1 and testing with D1) defined STableName as
  array[0..SizeOf(TFileName) - 1] of Char. In D1 TFileName is defined as
  String[79], but in D2 it's just string (= AnsiString = pointer),
  so sizeOf(TFileName) returns 4 instead, causing every filename truncated to
  first 3 chars. sorry.

******************************************************************************)

{$I dnInc.INC}

interface

uses
  Windows, SysUtils, Messages, Classes, Controls, Forms, Graphics, Menus,
  StdCtrls, Buttons, DB, DBCtrls,
  dnPxMask, dnPxFlds;

type
  TDBPxMask = class(TControlPxMask)
  private
    function GetFieldMask: TPxMask;
    function GetDataLink: TFieldDataLink;
    function GetField: TField;
  protected
    procedure SetModified(Value: Boolean); override;
    function CanEditModify: Boolean; override;
    procedure Reset; override;
    function MaskLoaded: Boolean; override;
    property DataLink: TFieldDataLink read GetDataLink;
    property Field: TField read GetField;
  public
    constructor create(aOwner: TWinControl);
  published
  end;

  TDBPxEdit = class(TDBEdit)
  private
    FPxMask: TDBPxMask;
    procedure SetPxMask(Value: TDBPxMask);
    function GetOnValidChanged: TValidChangedEvent;
    procedure SetOnValidChanged(Value: TValidChangedEvent);
  protected
  public
    constructor create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property PxMask: TDBPxMask read FPxMask write SetPxMask;
    property OnValidChanged: TValidChangedEvent read GetOnValidChanged
      write SetOnValidChanged
      stored false;
  end;

  TDBPxComboBox = class(TDBComboBox)
  private
    FPxMask: TDBPxMask;
    procedure SetPxMask(Value: TDBPxMask);
    function GetOnValidChanged: TValidChangedEvent;
    procedure SetOnValidChanged(Value: TValidChangedEvent);
  protected
    procedure KeyPress(var Key: Char); override;
  public
    constructor create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property PxMask: TDBPxMask read FPxMask write SetPxMask;
    property OnValidChanged: TValidChangedEvent read GetOnValidChanged
      write SetOnValidChanged
      stored false;
  end;

implementation
uses
  dnUtil, TypInfo;

type
  TPxFieldDataLink = class(TFieldDataLink)
  protected
    procedure RecordChanged(Field: TField); override;
    procedure UpdateData; override;
  end;

{$WARNINGS Off}
{$HINTS Off}
{*** This class-definition is only needed to access a private Var:
  FModified}
  TDummyFieldDataLink = class(TDataLink)
  private
    FField: TField;
    FFieldName: string;
    FControl: TComponent;
    FEditing: Boolean;
    FModified: Boolean;
  end;
{$WARNINGS On}
{$HINTS On}

procedure TPxFieldDataLink.RecordChanged(Field: TField);
var
  pxm: TPxMask;
begin
  inherited;
  if ((Field = nil) or (Field = self.Field)) and HasPxMask(Control, pxm) then
    pxm.LoadMask;
end;

procedure TPxFieldDataLink.UpdateData;
var
  pxm: TPxMask;
  OldState: TPxMaskState;
begin
  if TDummyFieldDataLink(self).FModified then
  begin
    if HasPxMask(Control, pxm) then
      with pxm as TDBPxMask do
      begin
        OldState := FState;
        LoadMask;
        FState := OldState;
        try
          if AutoValidate then ValidateEdit;
        except
          SetCaret;
          raise;
        end;
        include(FState, pxReEnter);
        inherited;
        exclude(FState, pxReEnter);
      end;
  end;
end;

{*** TDBPxMask ***}
constructor TDBPxMask.create(aOwner: TWinControl);
begin
  inherited create(aOwner, false);
  FakeInheritance(DataLink, TPxFieldDataLink);
end;

function TDBPxMask.GetDataLink: TFieldDataLink;
begin
  result := TFieldDataLink(TControl(Owner).Perform(CM_GETDATALINK, 0, 0));
end;

function TDBPxMask.GetField: TField;
begin
  with DataLink do
  begin
    result := Field;
    if (result = nil) and (not Active) and assigned(DataSet) then
      result := DataSet.FindField(FieldName);
  end;
end;

procedure TDBPxMask.SetModified(Value: Boolean);
begin
  inherited;
  if Value then DataLink.Modified;
end;

function TDBPxMask.GetFieldMask: TPxMask;
begin
  HasPxMask(Field, result);
end;

function TDBPxMask.MaskLoaded: Boolean;
var
  pxm: TPxMask;
begin
  if not (pxStored in FState) then Clear;
  pxm := GetFieldMask;
  result := assigned(pxm);
  if result and not pxm.Cleared then
    assign(pxm);
end;

function TDBPxMask.CanEditModify: Boolean;
begin
  result := DataLink.Edit;
end;

procedure TDBPxMask.Reset;
begin
  DataLink.Reset;
  SendMessage(EditHandle, EM_SETSEL, 0, -1);
  inherited;
end;

{*** TDBPxEdit ***}
constructor TDBPxEdit.create(aOwner: TComponent);
begin
  inherited create(aOwner);
  FPxMask := TDBPxMask.create(self); //MUST be created after inherited
end;

destructor TDBPxEdit.Destroy;
begin
  if assigned(FPxMask) then FPxMask.free;
  inherited Destroy;
end;

function TDBPxEdit.GetOnValidChanged: TValidChangedEvent;
begin
  result := FPxMask.OnValidChanged;
end;

procedure TDBPxEdit.SetOnValidChanged(Value: TValidChangedEvent);
begin
  FPxMask.OnValidChanged := Value;
end;

procedure TDBPxEdit.SetPxMask(Value: TDBPxMask);
begin
  FPxMask.assign(Value);
end;

{*** TDBPxComboBox ***}
constructor TDBPxComboBox.create(aOwner: TComponent);
begin
  inherited create(aOwner);
  FPxMask := TDBPxMask.create(self); //MUST be created after inherited
end;

destructor TDBPxComboBox.Destroy;
begin
  if assigned(FPxMask) then FPxMask.free;
  inherited Destroy;
end;

procedure TDBPxComboBox.KeyPress(var Key: Char);
begin
  {ComboBoxes don't get WM_CHAR messages (they're redirected to the EditHandle),
   so you've to do handle them in KeyPress}
  FPxMask.CheckResetKey(Key);
  FPxMask.HookKeyPress(Key);
  inherited KeyPress(Key);
end;

procedure TDBPxComboBox.SetPxMask(Value: TDBPxMask);
begin
  FPxMask.assign(Value);
end;

function TDBPxComboBox.GetOnValidChanged: TValidChangedEvent;
begin
  result := FPxMask.OnValidChanged;
end;

procedure TDBPxComboBox.SetOnValidChanged(Value: TValidChangedEvent);
begin
  FPxMask.OnValidChanged := Value;
end;

end.
